/**************************************************************
	
	boardclass.cpp: functions for handling the cribbage board
	Copyright 2002, Jacob A. Ewing

	Author: Jacob A. Ewing
	e-mail:	jacob@diagrami.com

**************************************************************

    This file is part of Netcrib.

    Netcrib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Netcrib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Netcrib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

**************************************************************/

#ifndef _STDIO_H
#	include <stdio.h>
#endif

#ifndef _IOSTREAM_H
#	include <string.h>
#endif

#ifndef __BOARDCLASS_H
#	include "boardclass.h"
#endif

holetype::holetype(){
	x = y = 0;
}

/***
Set the coordiantes of the hole.
***/
void holetype::set(int newy, int newx){
	x = newx;
	y = newy;
}


boardtype::boardtype(){
	xsize = ysize = numholes = numplayers = numcolourpairs = 0;
	image = colour = colourpair = (char **)NULL;
	hole = NULL;
	pegcolour = (int *)NULL;
}

boardtype::boardtype(int newysize, int newxsize, int length, int newnumplayers, int newnumcolourpairs){
	initialize(newysize, newxsize, length, newnumplayers, newnumcolourpairs);
}

/****
Initialize values.
****/
void boardtype::initialize(int newysize, int newxsize, int length, int newnumplayers, int newnumcolourpairs){

	int n;
	ysize = newysize;
	xsize = newxsize;

	pegcolour = new int[newnumplayers];
	image = new char*[newysize];
	colour = new char*[newysize];
	colourpair = new char*[newnumcolourpairs];

	numcolourpairs = newnumcolourpairs;
	for(n = 0; n < numcolourpairs; n++){
		colourpair[n] = new char[2];
	}

	for(n = 0; n < newysize; n++){
		image[n] = new char[newxsize + 1];
		colour[n] = new char[newxsize + 1];
		memset(image[n], ' ', (newxsize + 1) * sizeof(unsigned char));
		memset(colour[n], 0, (newxsize + 1) * sizeof(char));
	}

	numholes = length;
	numplayers = newnumplayers;

	hole = new holetype*[numplayers];
	for(n = 0; n < numplayers; n++){
		hole[n] = new holetype[numholes];
	}

}

/*****
Free alocated memory
*****/
boardtype::~boardtype(){
	delete hole;
	delete image;
	delete colour;
	delete colourpair;
	delete pegcolour;
}

/*****
save the board to the file named by *filename.  Returns 0 upon success.
*****/
int boardtype::save(char *filename){
	FILE *fout;
	int n, m;

	fout = fopen(filename, "wb");
	if(fout != NULL){
		fwrite((void *)&handsize, sizeof(int), 1, fout);
		fwrite((void *)&add_to_crib_size, sizeof(int), 1, fout);
		fwrite((void *)&starting_handsize, sizeof(int), 1, fout);
		fwrite((void *)&starting_cribsize, sizeof(int), 1, fout);
		fwrite((void *)&xsize, sizeof(int), 1, fout);
		fwrite((void *)&ysize, sizeof(int), 1, fout);
		fwrite((void *)&numholes, sizeof(int), 1, fout);
		fwrite((void *)&numplayers, sizeof(int), 1, fout);
		fwrite((void *)&numcolourpairs, sizeof(int), 1, fout);

		for(n = 0; n < ysize; n++)
			fwrite((void *)image[n], sizeof(char), xsize + 1, fout);
		for(n = 0; n < ysize; n++)
			fwrite((void *)colour[n], sizeof(char), xsize + 1, fout);
		for(n = 0; n < numcolourpairs; n++){
			fwrite((void *)colourpair[n], sizeof(char), 2, fout);
		}

		for(m = 0; m < numplayers; m++){
			for(n = 0; n < numholes; n++){
				fputc((int)hole[m][n].x, fout);
				fputc((int)hole[m][n].y, fout);
			}
			fputc((int)pegcolour[m], fout);
		}
		fputc((int)endhole.x, fout);
		fputc((int)endhole.y, fout);

		fclose(fout);
	}else{
		printf("Error opening save file.\n");
		return 1;
	}
	return 0;
}

/*****
Load the board from a file named by *filename.  Returns 0 upon success.
*****/
int boardtype::load(char *filename){
	FILE *fin;
	int n, m;
	int xs, ys, nh, np, nc;

	fin = fopen(filename, "rb");
	if(fin != NULL){
		fread((void *)&handsize, sizeof(int), 1, fin);
		fread((void *)&add_to_crib_size, sizeof(int), 1, fin);
		fread((void *)&starting_handsize, sizeof(int), 1, fin);
		fread((void *)&starting_cribsize, sizeof(int), 1, fin);
		fread((void *)&xs, sizeof(int), 1, fin);
		fread((void *)&ys, sizeof(int), 1, fin);
		fread((void *)&nh, sizeof(int), 1, fin);
		fread((void *)&np, sizeof(int), 1, fin);
		fread((void *)&nc, sizeof(int), 1, fin);

		initialize(ys, xs, nh, np, nc);

		for(n = 0; n < ysize; n++)
			fread((void *)image[n], sizeof(char), xsize + 1, fin);
		for(n = 0; n < ysize; n++)
			fread((void *)colour[n], sizeof(char), xsize + 1, fin);
		for(n = 0; n < numcolourpairs; n++){
			fread((void *)colourpair[n], sizeof(char), 2, fin);
		}

		for(m = 0; m < numplayers; m++){
			for(n = 0; n < numholes; n++){
				hole[m][n].x = fgetc(fin);
				hole[m][n].y = fgetc(fin);
			}
			pegcolour[m] = fgetc(fin);
		}
		endhole.x = fgetc(fin);
		endhole.y = fgetc(fin);

		fclose(fin);

	}else{
		printf("Error opening load file.\n");
		return 1;
	}
	return 0;
}
