/*****************************************************************************
	When done, this will hopefully be a nice network game with which
	you can play cribbage against other people or against the computer
	itself.  My starting goal is a simple console game where you can
	play cribbage against the computer.  

	The basic console game is finished now.  I'm now going to attempt
	to make it into a network game.  After that I'll be adding a nice GUI
*****************************************************************************/
#include <iostream.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <stdio.h>
//#include <termio.h>
#include <ncurses.h>

/**
an odd number here:  MAXHANDSIZE is set to the largest possible number of 
cards that can be played before hitting 31 when pegging.  This is because
this program handles pegging by duplicating them in another hand named
"pile" and checking for points in that.
**/
#define ENTER_KEY (char)10
#define decksize 52
#define MAXHANDSIZE 13
#define maxplayers 4
#define MAXNAMELENGTH 16
#define NUMCARDBACKS 2
#define NUMCARDFACES 52

#define JACK 11
#define QUEEN 12
#define KING 13
#define ACE 1

#define CARDBACK_COLOUR 5

/****************** enumerations *******************/
enum playmodetype{ HAND, CRIB, PEG, NOCUT };

/***** important note ******
The order in which the suits are listed in "suittype" is important.
This is because the images are loaded by a function that generates
them in that order.
***************************/
enum suittype{ Spades, Hearts, Clubs, Diamonds, nil };
enum displaymodetype { none, ansi };

/****************** classes *******************/
class holetype{
    public:
	int x, y;

	holetype(){
		x = y = 0;
	}

	void set(int newy, int newx){
		x = newx;
		y = newy;
	}
};


class boardtype{
    public:
        int xsize, ysize, numholes, numplayers, numcolourpairs, *pegcolour;
	char **image, **colour, **colourpair;
	holetype **hole, endhole;

	boardtype(){
		xsize = ysize = numholes = numplayers = numcolourpairs = 0;
		image = colour = colourpair = (char **)NULL;
		hole = NULL;
		pegcolour = (int *)NULL;
	}

	boardtype(int newysize, int newxsize, int length, int newnumplayers, int newnumcolourpairs){
		initialize(newysize, newxsize, length, newnumplayers, newnumcolourpairs);
	}

	void initialize(int newysize, int newxsize, int length, int newnumplayers, int newnumcolourpairs){

		int n;
		ysize = newysize;
		xsize = newxsize;

		pegcolour = new int[newnumplayers];
		image = new char*[newysize];
		colour = new char*[newysize];
		colourpair = new char*[newnumcolourpairs];

		numcolourpairs = newnumcolourpairs;
		for(n = 0; n < numcolourpairs; n++){
			colourpair[n] = new char[2];
		}

		for(n = 0; n < newysize; n++){
			image[n] = new char[newxsize + 1];
			colour[n] = new char[newxsize + 1];
			memset(image[n], ' ', (newxsize + 1) * sizeof(unsigned char));
			memset(colour[n], 0, (newxsize + 1) * sizeof(char));
		}

		numholes = length;
		numplayers = newnumplayers;

		hole = new holetype*[numplayers];
		for(n = 0; n < numplayers; n++){
			hole[n] = new holetype[numholes];
		}

	}

	~boardtype(){
		delete hole;
		delete image;
		delete colour;
		delete colourpair;
		delete pegcolour;
	}

	int save(char *filename){
		FILE *fout;
		int n, m;

		fout = fopen(filename, "wb");
		if(fout != NULL){
			fwrite((void *)&xsize, sizeof(int), 1, fout);
			fwrite((void *)&ysize, sizeof(int), 1, fout);
			fwrite((void *)&numholes, sizeof(int), 1, fout);
			fwrite((void *)&numplayers, sizeof(int), 1, fout);
			fwrite((void *)&numcolourpairs, sizeof(int), 1, fout);

			for(n = 0; n < ysize; n++)
				fwrite((void *)image[n], sizeof(char), xsize + 1, fout);
			for(n = 0; n < ysize; n++)
				fwrite((void *)colour[n], sizeof(char), xsize + 1, fout);
			for(n = 0; n < numcolourpairs; n++){
				fwrite((void *)colourpair[n], sizeof(char), 2, fout);
			}

			for(m = 0; m < numplayers; m++){
				for(n = 0; n < numholes; n++){
					fputc((int)hole[m][n].x, fout);
					fputc((int)hole[m][n].y, fout);
				}
				fputc((int)pegcolour[m], fout);
			}
			fputc((int)endhole.x, fout);
			fputc((int)endhole.y, fout);

			fclose(fout);

		}else{
			printf("Error opening save file.\n");
			return 1;
		}
		return 0;
	}

	int load(char *filename){
		FILE *fin;
		int n, m;
		int xs, ys, nh, np, nc;

		fin = fopen(filename, "rb");
		if(fin != NULL){
			fread((void *)&xs, sizeof(int), 1, fin);
			fread((void *)&ys, sizeof(int), 1, fin);
			fread((void *)&nh, sizeof(int), 1, fin);
			fread((void *)&np, sizeof(int), 1, fin);
			fread((void *)&nc, sizeof(int), 1, fin);

			initialize(ys, xs, nh, np, nc);

			for(n = 0; n < ysize; n++)
				fread((void *)image[n], sizeof(char), xsize + 1, fin);
			for(n = 0; n < ysize; n++)
				fread((void *)colour[n], sizeof(char), xsize + 1, fin);
			for(n = 0; n < numcolourpairs; n++){
				fread((void *)colourpair[n], sizeof(char), 2, fin);
			}

			for(m = 0; m < numplayers; m++){
				for(n = 0; n < numholes; n++){
					hole[m][n].x = fgetc(fin);
					hole[m][n].y = fgetc(fin);
				}
				pegcolour[m] = fgetc(fin);
			}
			endhole.x = fgetc(fin);
			endhole.y = fgetc(fin);

			fclose(fin);

		}else{
			printf("Error opening load file.\n");
			return 1;
		}
		return 0;
	}

	void draw(int starty, int startx){
		int x, y;
		for(y = 0; y < ysize; y++){
			move(starty + y, startx);
			for(x = 0; x < xsize; x++){
				attron(COLOR_PAIR(colour[y][x]));
				addch(image[y][x]);
				attroff(COLOR_PAIR(colour[y][x]));
			}
		}
	}
};

class cardtype{
    private:
	void displaysuit(void){
		switch(suit){
			case Diamonds: printw("D"); break;
			case Clubs: printw("C"); break;
			case Hearts: printw("H"); break;
			case Spades: printw("S"); break;
			default: printw("E");
		}
	}

	void displayvalue(void){
		if(value < 11 && value > 1)
			printw("%i", value);
		else{
			switch(value){
				case 1: printw("A"); break;
				case 11: printw("J"); break;
				case 12: printw("Q"); break;
				case 13: printw("K"); break;
				default: printw("E");
			}
		}
	}

    public:
	suittype suit;
	int pointvalue;
	int value;

	cardtype(){ erase(); }

	void erase(void){
		suit = nil;
		value = '0';
		pointvalue = 0;
	}


	void display(displaymodetype displaymode){
		int x, y, colournum;
		switch(displaymode){
		    case ansi:
			getyx(stdscr, y, x);
			if(suit == Hearts || suit == Diamonds)
				colournum = 2;
			else
				colournum = 1;

			move(y, x);
			attron(COLOR_PAIR(colournum));
			displayvalue();
			printw(" ");
			if(value != 10) printw(" ");

			move(y + 1, x);
			printw(" ");
			displaysuit();
			printw(" ");

			move(y + 2, x);
			printw(" ");
			if(value != 10) printw(" ");
			displayvalue();

			attroff(COLOR_PAIR(colournum));
		}
	}

	void display(int y, int x){
		int colournum;
		if(suit == Hearts || suit == Diamonds)
			colournum = 2;
		else
			colournum = 1;

		move(y, x);
		attron(COLOR_PAIR(colournum));
		displayvalue();
		printw(" ");
		if(value != 10) printw(" ");

		move(0, 0);
		move(y + 1, x);
		printw(" ");
		displaysuit();
		printw(" ");

		move(0, 0);
		move(y + 2, x);
		printw(" ");
		if(value != 10) printw(" ");
		displayvalue();

		attroff(COLOR_PAIR(colournum));
	}

	cardtype operator = (cardtype newcard){
		suit = newcard.suit;
		value = newcard.value;
		pointvalue = newcard.pointvalue;
		return newcard;
	}
};

class handtype{
    public:
	cardtype card[MAXHANDSIZE];
	int numcards;

	handtype(){
		/*** start with an empty hand ***/
		numcards = 0;
	}

	void erase(void){
		int n;
		for(n = 0; n < MAXHANDSIZE; n++){
			card[n].erase();
		}
		numcards = 0;
	}

	void display(displaymodetype displaymode){
		int n;
		for(n = 0; n < numcards; n++){
			card[n].display(displaymode);
		}
	}

	void display(int y, int x){
		int n;
		for(n = 0; n < numcards; n++){
			card[n].display(y, x + (n << 2));
		}
	}


	int sum(void){
		int n, returnval = 0;
		for(n = 0; n < numcards; n++){
			returnval += card[n].pointvalue;
		}

		return returnval;
	}

	handtype operator = (handtype newhand){
		int n;
		erase();
		numcards = newhand.numcards;
		for(n = 0; n < numcards; n++){
			card[n] = newhand.card[n];
		}
		return newhand;
	}

	cardtype operator += (cardtype newcard){
		if(numcards < MAXHANDSIZE){
			card[numcards] = newcard;
			numcards++;
		}
		return newcard;
	}

	void sort(void){
		cardtype tempcard;
		int n, m;
		for(n = 1; n < numcards; n++){
			m = n;
			/*** Move the value down to the appropriate spot. ***/
			while(m > 0 && card[m].value < card[m - 1].value){
				/*** switch the values around ***/
				tempcard = card[m];
				card[m] = card[m - 1];
				card[m - 1] = tempcard;
				m--;
			}
		}
	}

	void passcard(handtype *recipient, int cardindex){
		*recipient += card[cardindex];
		removecard(cardindex);
	}

	void removecard(int cardindex){
		int n;
		numcards--;
		for(n = cardindex; n < numcards; n++){
			card[n] = card[n + 1];
		}
		card[n].erase();
	}

};

int count_play_value(handtype pile, int current_player, displaymodetype displaymode);

class playertype{
    private:
	void divide_by_human(void);
	void divide_by_computer(void);

    public:
	int score, priorscore, typeofplayer;
	handtype hand;
	char name[MAXNAMELENGTH];

	playertype(){
		score = priorscore = typeofplayer = 0;
	}

	void getname(void){
		if(typeofplayer == 1 || 1 /********** this "if" needs to be changed later **************/){
			switch(rand() % 26){
				case 0: strcpy(name, "Alfred"); break;
				case 1: strcpy(name, "Bonnie"); break;
				case 2: strcpy(name, "Charles"); break;
				case 3: strcpy(name, "Dominique"); break;
				case 4: strcpy(name, "Edward"); break;
				case 5: strcpy(name, "Francine"); break;
				case 6: strcpy(name, "Ginny"); break;
				case 7: strcpy(name, "Howard"); break;
				case 8: strcpy(name, "Ivan"); break;
				case 9: strcpy(name, "Josephine"); break;
				case 10: strcpy(name, "Kenny"); break;
				case 11: strcpy(name, "Laurel"); break;
				case 12: strcpy(name, "Michael"); break;
				case 13: strcpy(name, "Nora"); break;
				case 14: strcpy(name, "Oliver"); break;
				case 15: strcpy(name, "Penelope"); break;
				case 16: strcpy(name, "Quincy"); break;
				case 17: strcpy(name, "Rebecca"); break;
				case 18: strcpy(name, "Samson"); break;
				case 19: strcpy(name, "Tasha"); break;
				case 20: strcpy(name, "Uriel"); break;
				case 21: strcpy(name, "Victoria"); break;
				case 22: strcpy(name, "William"); break;
				case 23: strcpy(name, "Xena"); break;
				case 24: strcpy(name, "Yolanda"); break;
				case 25: strcpy(name, "Zachary"); break;
			}
		}
	}

	void showhand(displaymodetype displaymode){
		hand.display(displaymode);
		cout << '\n';
	}

	void divide_hand(void){
		if(typeofplayer == 0) divide_by_human();
		else divide_by_computer();
	}

	int play_card(handtype *pile);
};

class decktype{
    public:
	cardtype card[decksize];
	int numcards;

	decktype(){
		int n;

		/*** first, create the cards ***/
		numcards = decksize;
		for(n = 0; n < decksize; n++){
			card[n].suit = (suittype)(n % 4);
			card[n].value = (n >> 2) + 1;
			card[n].pointvalue = card[n].value;
			if(card[n].pointvalue > 10) card[n].pointvalue = 10;
		}

		/*** now shuffle them ***/
		shuffle();
	}

	void shuffle(void){
		int n, m, numruns;
		cardtype swapcard;

		for(numruns = 0; numruns < 6; numruns++){
			for(n = 0; n < decksize; n++){

				if(rand() % 2){
					swapcard = card[n];
					for(m = n; m > 0; m--)
						card[m] = card[m - 1];
					card[0] = swapcard;
				}
			}
		}
	}

	void display(displaymodetype displaymode){
		int n;
		for(n = 0; n < numcards; n++){
			card[n].display(displaymode);
			if(n % 4 == 3) cout << '\n';
		}
	}

	void deal(playertype *p, int numtodeal){
		int n;
		for(n = 0; n < numtodeal; n++){
			if(p->hand.numcards < MAXHANDSIZE && numcards > 0){
				numcards--;
				p->hand.card[p->hand.numcards] = card[numcards];
				card[numcards].erase();
				p->hand.numcards++;
			}
		}
	}

	cardtype draw(int cutpos){
		cardtype returnval;
		int n;
		if(cutpos >= 0 && cutpos < numcards){
			returnval = card[cutpos];
			numcards--;
			for(n = cutpos; n < numcards; n++){
				card[n] = card[n + 1];
			}
			card[n].erase();
		}
		return returnval;
	}

	cardtype operator += (cardtype addition){
		if(numcards < decksize){
			card[numcards] = addition;
			numcards ++;
		}
		return addition;
	}

	handtype operator += (handtype addition){
		int n;
		if(numcards + addition.numcards <= decksize){
			for(n = 0; n < addition.numcards; n++){
				card[numcards] = addition.card[n];
				numcards ++;
			}
		}
		return addition;
	}
};


/****************** function declarations *********************/
void randomize(void);
int initialize_table(int argc, char *argv[]);
void deal_round(void);
int count_points(handtype hand, playmodetype playmode, displaymodetype displaymode);
int count_fifteens(handtype hand, playmodetype playmode, displaymodetype displaymode);
int count_runs(handtype hand, playmodetype playmode, displaymodetype displaymode);
int count_pairs(handtype hand, playmodetype playmode, displaymodetype displaymode);
int count_flush(handtype hand, playmodetype playmode, displaymodetype displaymode);
int count_nobs(handtype hand, playmodetype playmode, displaymodetype displaymode);
void divide_hands(void);
void cut_deck(void);
void play_hands(void);
void peg_hands(displaymodetype displaymode);
void show_winner(displaymodetype displaymode);
void get_player_names(void);
void display_round_start(int numrounds);
void restock_deck(void);
void draw_cardback(int y, int x);
void draw_pegs(void);
void box(int y1, int x1, int y2, int x2, char value);

/****************** global variables ******************/
playertype player[maxplayers];
decktype deck;
cardtype cutcard;
int numplayers, handsize, starting_handsize, starting_cribsize, add_to_crib_size, current_dealer;
int game_over, winning_player, maximum_score;
displaymodetype main_displaymode;
handtype crib, stack;
int current_playmode = 0, xscreensize, yscreensize;
boardtype board;
int numcolourpairs, baseboardcolour, boardx, boardy, messagex, messagey;

/****************** function definitions *********************/
int main(int argc, char *argv[]){
	int n, x, y, returnval;//, numrounds;
	displaymodetype displaymode;

	randomize();

	returnval = initialize_table(argc, argv);
	if(returnval) exit(returnval);

	for(n = 1; n < numplayers; n++) player[n].typeofplayer = 1;
	player[0].typeofplayer = 0;

	displaymode = main_displaymode;

	/****** initialize screen ******/
	initscr();
	getmaxyx(stdscr, yscreensize, xscreensize);

	if(xscreensize < 80 || yscreensize < 25){
		endwin();
		cout << "Error: Screen must be a minimum size of 80 X 25.\n";
		exit(1);
	}

	noecho();
	//raw();
	curs_set(0);
	clear();


	/********* initialize colours *********/
	start_color();
	init_pair(1, COLOR_BLACK, COLOR_WHITE);
	init_pair(2, COLOR_RED, COLOR_WHITE);
	init_pair(3, COLOR_WHITE, COLOR_BLACK);
	init_pair(4, COLOR_BLACK, COLOR_YELLOW);
	init_pair(CARDBACK_COLOUR, COLOR_GREEN, COLOR_BLUE);
	numcolourpairs = 5;
	//bkgd(COLOR_PAIR(3));

	/********** load the board and initialize related values **********/
	if(board.load("default.board")) exit(1);

	baseboardcolour = numcolourpairs + 1;
	for(n = 0; n < board.numcolourpairs; n++){
		numcolourpairs++;
		init_pair(numcolourpairs, board.colourpair[n][0], board.colourpair[n][1]);
	}

	for(y = 0; y < board.ysize; y++){
		for(x = 0; x < board.xsize; x++){
			board.colour[y][x] += baseboardcolour;
		}
	}

	for(n = 0; n < board.numplayers; n++){
		board.pegcolour[n] += baseboardcolour;
	}

	boardy = (yscreensize - board.ysize) >> 1;
	boardx = xscreensize - board.xsize - 1;

	/********** initialize any needed windows ***********/
	messagey = boardy + board.ysize;
	messagex = boardx;

	/********** play the game ************/
	get_player_names();
	deck.shuffle();

	clear();
	board.draw(boardy, boardx);
	deal_round();
	divide_hands();
	cut_deck();
	play_hands();
	do{}while(getch() != (char)27);
/*
	clear();
	board.draw(boardx, boardy);
	cut_deck();
	play_hands(displaymode);
*/


/*************
	get_player_names();

	set_keypress();
	numrounds = 0;
	while(!game_over){
		numrounds ++;
		display_round_start(numrounds);
		deck.shuffle();
		deal_round();
		divide_hands();
		cut_deck(displaymode);
		if(!game_over) play_hands(displaymode);
		if(!game_over) peg_hands(displaymode);

		restock_deck();

	       	if(displaymode && !game_over){
			cout << "Press a key for the next round";
			getchar();
		}
		current_dealer = (current_dealer + 1) % numplayers;
	}
	show_winner(displaymode);
	reset_keypress();
**************/

	endwin();

	return returnval;
}

void draw_pegs(void){
	int n, x, y;
	for(n = 0; n < numplayers; n++){
		if(player[n].score > 0){
			attron(COLOR_PAIR(board.pegcolour[n]));
			if(player[n].score < 121){

				x = board.hole[n][player[n].score - 1].x;
				y = board.hole[n][player[n].score - 1].y;
				move(y + boardy, x + boardx);
				addch('!');

				x = board.hole[n][player[n].priorscore - 1].x;
				y = board.hole[n][player[n].priorscore - 1].y;
				move(y + boardy, x + boardx);
				addch('!');

			}else{
				x = board.endhole.x;
				y = board.endhole.y;
				move(y + boardy, x + boardx);
				addch('!');

				x = board.hole[n][player[n].priorscore - 1].x;
				y = board.hole[n][player[n].priorscore - 1].y;
				move(y + boardy, x + boardx);
				addch('!');

			}
			attroff(COLOR_PAIR(board.pegcolour[n]));
		}

	}
}

void restock_deck(void){
	int n;
	for(n = 0; n < numplayers; n++){
		deck += player[n].hand;
		player[n].hand.erase();
	}
	deck += crib;
	crib.erase();
	deck += cutcard;
	cutcard.erase();
}

void display_round_start(int numrounds){
	cout << "\n\n============================== Round " << numrounds << " ==============================\n";
	cout << player[current_dealer].name << " deals.\n";
}

void get_player_names(void){
	int n, m;
	for(n = 0; n < numplayers; n++){
		if(player[n].typeofplayer == 1){
			do{
				player[n].getname();
				for(m = 0; m < n && strcmp(player[n].name, player[m].name); m++);
			}while(m < n);
		}else{
			move(0, 0);
			curs_set(1);
			printw("Hi there player %i, what is your name? ", n + 1);
			echo();
			scanw("%s", player[n].name);
			noecho();
			curs_set(0);
			clear();
		}
	}
}

void show_winner(displaymodetype displaymode){
	if(displaymode != none){
		cout << player[winning_player].name << " wins!\n";
	}

}

void draw_cardback(int y, int x){
	move(y, x);
	attron(COLOR_PAIR(CARDBACK_COLOUR));
	printw("\\|/");
	move(y + 1, x);
	printw("-X-");
	move(y + 2, x);
	printw("/|\\");
	attroff(COLOR_PAIR(CARDBACK_COLOUR));	
}

int initialize_table(int argc, char *argv[]){
	int n, returnval = 0, temp;
	/*** set the default values ***/
	numplayers = 2;					/* The number of players */
	handsize = 4;					/* The size of each player's hand after putting cards in the crib */
	starting_handsize = 6;				/* The number of cards initially dealt to each player */
	starting_cribsize = 0;				/* The number of cards initially dealt into the crib */
	current_dealer = rand() % numplayers;		/* The index number of the player who's dealing */
	add_to_crib_size = 2;				/* The number of cards each player needs to throw in the crib */
	maximum_score = 120;				/* The number of points required to win the game */
	game_over = 0;					/* non-zero if the game has been won */
	main_displaymode = ansi;			/* primary mode of display */

	if(argc > 1){
		for(n = 1; n < argc; n++){
			if(!strcmp(argv[n], "-f")){
				if(argc > n + 1){
					maximum_score = atoi(argv[n + 1]);
					n++;
				}else{
					returnval = 1;
					break;
				}
			}else if(!strcmp(argv[n], "-p")){
				if(argc > n + 1){
					temp = atoi(argv[n + 1]);
					switch(temp){
					    case 2:
						starting_handsize = 6;
						add_to_crib_size = 2;
						starting_cribsize = 0;
						numplayers = temp;
						break;
					    case 3:
						starting_handsize = 5;
						add_to_crib_size = 1;
						starting_cribsize = 1;
						numplayers = temp;
						break;
					    case 4:
						starting_handsize = 5;
						add_to_crib_size = 1;
						starting_cribsize = 0;
						numplayers = temp;
						break;
					}
					n++;
				}else{
					returnval = 1;
					break;
				}
			}else if(!strcmp(argv[n], "-s")){
				if(argc > n + 1){
					current_dealer = atoi(argv[n + 1]);
					n++;
				}else{
					returnval = 1;
					break;
				}
			}else if(!strcmp(argv[n], "-d")){
				if(argc > n + 1){
					n++;
					if(!strcmp(argv[n], "none")) main_displaymode = none;
					if(!strcmp(argv[n], "ansi")) main_displaymode = ansi;
				}else{
					returnval = 1;
					break;
				}
			}
		}
	}

	if(returnval == 1){
		cout << "Command Syntax:\n";
		cout << "cribbage [-f (finishing score)]\n";
		cout << "         [-p (number of players)]\n";
		cout << "         [-s (starting dealer)]\n";
		cout << "         [-d (display mode)]\n\n";
		cout << "\"display mode\" can be one of the following:\n";
		cout << "\tnone: Will not display anything.\n";
		cout << "\tansi: Displays the entire game in ansi highlighted text.\n";
	}

	return returnval;
}

void peg_hands(displaymodetype displaymode){
	int n, m;
	for(n = 0; n < numplayers; n++){

		m = (n + current_dealer + 1) % numplayers;
		if(displaymode == ansi){
			cout << '\n' << player[m].name << " pegs:\n";
			player[m].hand.display(displaymode);
			cout << "   "; cutcard.display(displaymode); cout << '\n';
		}
		player[n].priorscore = player[m].score;
		player[m].score += count_points(player[m].hand, HAND, displaymode);

		if(displaymode == ansi){
			cout << player[m].name << "'s score now: ";
			cout << player[m].score << "\n";
		}

		if(player[m].score >= maximum_score){
			game_over = 1;
			winning_player = m;
			break;
		}

	}

	if(!game_over){
		if(displaymode == ansi){
			cout << "And in the crib:\n";
			crib.display(displaymode);
			cout << "   "; cutcard.display(displaymode); cout << '\n';
		}

		m = count_points(crib, CRIB, displaymode);
		if(m){
			player[current_dealer].priorscore = player[current_dealer].score;
			player[current_dealer].score += m;
		}

		if(displaymode == ansi){
			cout << player[current_dealer].name << "'s score now: ";
			cout << player[current_dealer].score << "\n";
		}

		if(player[current_dealer].score >= maximum_score){
			game_over = 1;
			winning_player = current_dealer;
		}
	}
}

int count_play_value(handtype pile, int current_player, displaymodetype displaymode){
	int n, m, sum, returnval = 0, testsize, runsize;
	handtype temphand;

	/******* check for fifteens or thirty-ones *******/
	sum = pile.sum();
	if(displaymode == ansi)
		cout << '\t' << sum << ' ';

	if(sum == 15){
		returnval += 2;
		if(displaymode == ansi)
			cout << " Fifteen two";
	}

	if(sum == 31){ 
		if(returnval > 0 && displaymode == ansi) cout << ',';
		returnval += 2;
		if(displaymode == ansi){
			cout << " Thirty-one for two";
		}
	}else if(pile.numcards == numplayers * handsize){
		if(returnval > 0 && displaymode == ansi) cout << ',';
		returnval++;
		if(displaymode == ansi){
			cout << " Go for one";
		}
	}

	/******* check for pairs, pair royals, etc. ********/
	m = 1;
	for(n = pile.numcards - 1; n > 0 && pile.card[n].value == pile.card[n - 1].value; n--) m++;
	if(m >= 2){
		if(returnval > 0 && displaymode == ansi) cout << ',';
		returnval += (m * (m - 1));

		if(displaymode == ansi){
			if(m == 2) cout << " pair for two";
			else cout << " pair royal for " << m * (m - 1);
		}
	}

	/******** check for runs *******/
	/** not yet certain that this works properly **/
	if(pile.numcards > 2){
		runsize = 0;
		temphand.erase();

		temphand += pile.card[pile.numcards - 1];
		temphand += pile.card[pile.numcards - 2];

		for(testsize = 3; testsize <= pile.numcards; testsize ++){
			temphand += pile.card[pile.numcards - testsize];
			temphand.sort();
			m = 1;
			for(n = 1; n < testsize; n++){
				if(temphand.card[n].value - temphand.card[n - 1].value == 1){
					m++;
				}else{
					break;
				}
			}
			if(n >= testsize){
				runsize = m;
			}
		}
		if(runsize > 2){
			if(returnval > 0 && displaymode == ansi) cout << ',';
			returnval += runsize;
			if(displaymode == ansi)
				cout << " Run for " << runsize;
		}
	}

	return returnval;
}

void play_hands(void){
	int total_cards_played = 0, current_numcards = 0, n;
	int current_player, last_player;
	handtype pile, original_hand[numplayers];

	for(n = 0; n < numplayers; n++){
		original_hand[n] = player[n].hand;
	}

	pile.erase();
	current_player = (current_dealer + 1) % numplayers;

	while(total_cards_played < numplayers * handsize && !game_over){
		if(player[current_player].play_card(&pile)){

			pile.card[current_numcards].display(yscreensize >> 1, current_numcards << 2);

			current_numcards++;
			total_cards_played ++;

			n = count_play_value(pile, current_player, ansi);
			if(n != 0){
				player[current_player].priorscore = player[current_player].score;
				player[current_player].score += n;
			}

			if(player[current_player].score >= maximum_score){
				game_over = 1;
				winning_player = current_player;

			}

			last_player = current_player;
		}else{
			if(last_player == current_player){
				pile.erase();
				current_numcards = 0;

				player[current_player].priorscore = player[current_player].score;
				player[current_player].score++;

				if(player[current_player].score >= maximum_score){
					game_over = 1;
					winning_player = current_player;
				}

				//if(displaymode == ansi){
					cout << '\n' << player[current_player].name;
					for(n = strlen(player[current_player].name); n < MAXNAMELENGTH + 2; n++) cout << ' ';
					cout << "\t\tGo for one";
				//}
			}
		}

		if(pile.sum() == 31){
			pile.erase();
			current_numcards = 0;
		}

		current_player = (current_player + 1) % numplayers;

	}

	if(pile.numcards && !game_over){
		player[last_player].priorscore = player[last_player].score;
		player[last_player].score ++;
		//if(displaymode == ansi){
			cout << " One for last card";
		//}
	}

	//if(displaymode == ansi)
		cout << '\n';

	for(n = 0; n < numplayers; n++){
		player[n].hand = original_hand[n];
	}

}

void deal_round(void){
	int n, m, l;

	/*** now deal the cards ***/
	for(n = 0; n < starting_handsize; n++){
		for(m = 0; m < numplayers; m++){
			l = (m + current_dealer) % numplayers;
			deck.deal(&player[l], 1);
		}
	}

	for(n = 0; n < starting_cribsize; n++){
		deck.numcards --;
		crib += deck.card[deck.numcards];
		deck.card[deck.numcards].erase();
	}
}

void cut_deck(void){
	int range, cutnum, minimum, x, y;
	minimum = 4;
	range = deck.numcards - 8;
	cutnum = minimum + rand() % range;
	cutcard = deck.draw(minimum + rand() % range);
	
	x = 4;
	y = (yscreensize >> 1) - 2;
	mvprintw(y, x, "Cut:");
	x = 5;
	y = (yscreensize >> 1) - 1;
	cutcard.display(y, x);
	draw_cardback(y, 0);

	if(cutcard.value == JACK){
		player[current_dealer].priorscore = player[current_dealer].score;
		player[current_dealer].score += 1;
		if(player[current_dealer].score > maximum_score){
			game_over = 1;
			winning_player = current_dealer;
		}
		x = 0;
		y = (yscreensize >> 1) + 3;
		mvprintw(y, x, "Dealer gets one for Nobs.");
	}
}

void randomize(void){
	srand(time(0));
}

void divide_hands(void){
	int n;
	for(n = 0; n < numplayers; n++){
		player[n].divide_hand();
	}
}

void box(int y1, int x1, int y2, int x2, char value){
	int y, xsize;
	char *string;

	xsize = x2 - x1 + 1;
	string = new char[xsize + 1];
	memset((void *)string, value, xsize);
	string[xsize] = (char)NULL;
	
	for(y = y1; y <= y2; y++){
		mvprintw(y, x1, string);
	}

	delete string;

}

int count_fifteens(handtype hand, playmodetype playmode, displaymodetype displaymode){
	int score, numcombos, tally, n, m;
	handtype temphand;

	temphand = hand;
	if(playmode == HAND || playmode == CRIB) temphand += cutcard;
	temphand.sort();

	/************ looking for fifteens. ***********/
	numcombos = 1 << temphand.numcards;   /* This is the total number of combinations of the cards */
	score = 0;

	for(n = 1; n < numcombos; n++){
		tally = 0;
		for(m = 0; m < temphand.numcards; m++){
			if((1 << m) & n){
				tally += temphand.card[m].pointvalue;
			}
			
		}

		if(tally == 15){

			if(score != 0 && displaymode == ansi) cout << ", ";
			score += 2;
			if(displaymode == ansi) cout << "Fifteen " << score;
		}
	}
	if(displaymode == ansi && score != 0) cout << '\n';

	return score;
}

int count_pairs(handtype hand, playmodetype playmode, displaymodetype displaymode){
	int n, m, alreadycounted, tally, score = 0;
	handtype temphand;

	temphand = hand;
	if(playmode == HAND || playmode == CRIB) temphand += cutcard;


	/********** looking for pairs and pair royals **********/
	for(n = 0; n < temphand.numcards; n++){
		/*** determine if the current card has already been counted in a pair combo ***/
		alreadycounted = 0;
		for(m = 0; m < n; m++){
			if(temphand.card[m].value == temphand.card[n].value)
				alreadycounted = 1;
		}

		if(!alreadycounted){
			tally = 1;
			for(m = n + 1; m < temphand.numcards; m++){
				if(temphand.card[n].value == temphand.card[m].value)
					tally++;
			}

			/** no if here, since tally - 1 == 0 if there are no pairs **/
			if(displaymode == ansi && tally > 1){
				if(score != 0) cout << ", ";
				switch(tally){
				    case 2: cout << "Pair for 2"; break;
				    case 3: cout << "Pair royal for 6"; break;
				    case 4: cout << "Double royal for 12"; break;
				}
			}
			score += tally * (tally - 1);
		}
	}
	if(displaymode == ansi && score != 0) cout << '\n';

	return score;
}

int count_runs(handtype hand, playmodetype playmode, displaymodetype displaymode){
	int n, runlength, multiplicand, newfactor, score, increment;
	handtype temphand;

	score = 0;
	runlength = multiplicand = newfactor = 1;

	temphand = hand;
	if(playmode == HAND || playmode == CRIB) temphand += cutcard;
	temphand.sort();

	for(n = 1; n < temphand.numcards; n++){
		if(temphand.card[n - 1].value == temphand.card[n].value){
			/*** if the current card is the same number as the previous***/
			newfactor ++;
		}else{
			if(newfactor > 1){
				multiplicand *= newfactor;
				newfactor = 1;
			}

			if(temphand.card[n - 1].value == temphand.card[n].value - 1){
				/*** if the curent card is one up from the previous **/
				runlength++;
			}else{
				if(runlength > 2){
					increment = runlength;
					increment *= multiplicand;
					score += increment;

					if(displaymode == ansi){
						switch(multiplicand){
						    case 2: cout << "Double run "; break;
						    case 3: cout << "Triple run "; break;
						    case 4: cout << "Quadruple run "; break;
						    case 5: cout << "Quintuple run "; break;
						    default: cout << "Run ";
						}
						cout << "for " << increment;
					}
				}
				multiplicand = newfactor = runlength = 1;
			}
		}
	}

	/***
	This repetition of the "if" needs to be done in case the loop
	ends with a run at the end of the hand.
	***/
	if(runlength > 2){
		if(newfactor > 1) multiplicand *= newfactor;
		increment = runlength;
		increment *= multiplicand;
		score += increment;

		if(displaymode == ansi){
			switch(multiplicand){
			    case 2: cout << "Double run "; break;
			    case 3: cout << "Triple run "; break;
			    case 4: cout << "Quadruple run "; break;
			    case 5: cout << "Quintuple run "; break;
			    default: cout << "Run ";
			}
			cout << "for " << increment;
		}
	}
	if(score > 0 && displaymode == ansi) cout << '\n';
	return score;
}

int count_flush(handtype hand, playmodetype playmode, displaymodetype displaymode){
	int score = 0, n, isaflush = 1;

	for(n = 1; n < hand.numcards; n++){
		if(hand.card[n].suit != hand.card[n - 1].suit)
			isaflush = 0;
	}

	if(isaflush){
		if(playmode == CRIB && cutcard.suit != hand.card[0].suit) isaflush = 0;
	}

	if(isaflush){
		score = hand.numcards;
		if(cutcard.suit == hand.card[0].suit) score++;
	}

	if(score != 0  && displaymode == ansi){
		cout << score << " card flush\n";
	}
	return score;
}

int count_nobs(handtype hand, playmodetype playmode, displaymodetype displaymode){
	int returnval = 0, n;

	if(cutcard.value != JACK && playmode != PEG){
		for(n = 0; n < hand.numcards; n++){
			if(hand.card[n].value == JACK
			   && hand.card[n].suit == cutcard.suit)
				returnval = 1;
		}
	}

	if(displaymode == ansi && returnval != 0){
		cout << "1 for his nobs\n";
	}

	return returnval;
}

int count_points(handtype hand, playmodetype playmode, displaymodetype displaymode){
	/***
	This goes through every possible type of score and checks
	to see if any of them can be found in the hand.  eg.  first
	it looks for anything that adds up to fifteen, then it looks
	for runs, and so on.  The individual checks are performed
	by separate functions.
	***/
	int score;

	score = count_fifteens(hand, playmode, displaymode);
	score += count_runs(hand, playmode, displaymode);
	score += count_pairs(hand, playmode, displaymode);
	score += count_nobs(hand, playmode, displaymode);
	score += count_flush(hand, playmode, displaymode);	

	/*** display the score ***/
	if(displaymode == ansi){
		if(score == 0)
			cout << "Good old \"19\".\n";
		else{
			cout << "for a total of " << score << ".\n";
		}
	}

	return score;
}



/*** 
This returns a 1 or 0 depending on whether or not it can play a card
without going over 31, and if so, it puts that card into the hand "pile".
it does NOT add to the score.  This is done elsewhere.
***/
int playertype::play_card(handtype *pile){
	int returnval = 0, n, bestscore = -1, newscore, bestplaycard = -1, tally, cardpicked;
	handtype testhand;
	int gotone;
	unsigned char keypressed;

	if(pile->numcards < MAXHANDSIZE){

		/*******************************************
		This part of the if handles User I/O if the
		player is a human one.
		*******************************************/
		if(typeofplayer == 0){
			if(hand.numcards > 0){

				/** first, display the cards and options for the player to see **/
				move(messagey, messagex);
				printw("Select which card to play %s.", name);
				move(messagey + 1, messagex);
				printw("Press \"p\" to pass)");
				for(n = 0; n < hand.numcards; n++){
					hand.card[n].display(yscreensize - 6, n * 4);
					move(yscreensize - 3, n * 4);
					printw("[%c] ", (char)((int)'a' + n));
				}
				refresh();

				gotone = 0;
				do{
					keypressed = getch();
					if(keypressed >= 'A' && keypressed <= 'Z') keypressed += (int)'a' - (int)'A';
					if(keypressed >= 'a' && keypressed < (char)((int)'a' + hand.numcards)) gotone = 1;
					if(keypressed == 'p') gotone = 1;

					/******* now check to see if it's a valid option ******/
					if(gotone){
						cardpicked = (int)(keypressed - 'a');
						tally = pile->sum();
						if(keypressed == 'p'){
							/**** make sure they actually can't play a card ****/
							for(n = 0; n < hand.numcards && (tally + hand.card[n].pointvalue > 31); n++);
							if(n < hand.numcards) gotone = 0;
						}else{
							/**** make sure the selected card can actually be played ****/
							newscore = tally + hand.card[cardpicked].pointvalue;
							if(newscore > 31) gotone = 0;
						}
					}
				}while(!gotone);

				if(keypressed != 'p'){
					returnval = 1;
					pile->card[pile->numcards] = hand.card[cardpicked];
					pile->numcards ++;
					hand.removecard(cardpicked);
					returnval = 1;
				}
			}

		/*******************************************
		This part of the if handles the computer player.
		*******************************************/
		}else{
			tally = pile->sum();
			for(n = 0; n < hand.numcards; n++){
				if(tally + hand.card[n].pointvalue <= 31){
					testhand = *pile;
					testhand += hand.card[n];
					newscore = count_play_value(testhand, 0, none);
					if(newscore > bestscore){
						bestplaycard = n;
						bestscore = newscore;
					}
				}
			}

			if(bestscore == 0){
				do{
					n = rand() % hand.numcards;
				}while(tally + hand.card[n].value > 31);
				bestplaycard = n;
			}

			if(bestplaycard != -1){
				pile->card[pile->numcards] = hand.card[bestplaycard];
				pile->numcards++;
				hand.removecard(bestplaycard);
				returnval = 1;
			}
		}
	}else{
		cout << "\n<<<<<< ERROR: Not enough space in the pile >>>>>>\n";
	}
	return returnval;
}


void playertype::divide_by_human(void){
	unsigned char keypressed;
	int n, m, done = 0;
	displaymodetype displaymode = main_displaymode;
	int numcardspicked = 0, card_picked[MAXHANDSIZE];

	if(add_to_crib_size){
		hand.sort();

		/** first, display the cards and options for the player to see **/
		if(displaymode == ansi){

			move(messagey, messagex);
			printw("Select which cards to put into %s's crib:", player[current_dealer].name);
			move(messagey + 1, messagex);
			printw("throw into the crib:");
			move(messagey + 2, messagex);
			printw("(press the space bar when ready)");

			for(n = 0; n < hand.numcards; n++){
				hand.card[n].display(yscreensize - 6, n * 4);
				move(yscreensize - 3, n * 4);
				printw("[%c] ", (char)((int)'a' + n));
			}
			refresh();
		}

		/*** now let them pick out the cards they pass to the crib ***/
		numcardspicked = 0;
		do{
			keypressed = getch();

			if(keypressed == (char)27){
				endwin();
				exit(1);
			}

			if(keypressed == ' '){
				if(numcardspicked != add_to_crib_size){
					move(messagey + 3, messagex);
					attron(A_BOLD);
					if(numcardspicked < add_to_crib_size)
						printw("You need to select %i cards. ", add_to_crib_size);
					else
						printw("You can only select %i cards.", add_to_crib_size);
					attroff(A_BOLD);
				}else{
					done = 1;
				}
			}

			for(n = 0; n < hand.numcards && (keypressed != (unsigned char)((int)'a' + n)); n++);
			if(n < hand.numcards){
				for(m = 0; m < numcardspicked; m++)
					if(card_picked[m] == n) break;

				if(m >= numcardspicked){
					card_picked[numcardspicked] = n;
					numcardspicked ++;

					attron(COLOR_PAIR(4));

					move(yscreensize - 3, 4 * n);
					printw("[%c]", 'A' + keypressed - 'a');
					attroff(COLOR_PAIR(4));

					//attron(A_BOLD);
					hand.card[n].display(yscreensize - 6, n * 4);
					//attroff(A_BOLD);

				}else{
					move(yscreensize - 3, 4 * n);
					printw("[%c]", keypressed);

					numcardspicked--;
					for(; m < numcardspicked; m++){
						card_picked[m] = card_picked[m + 1];
					}					
				}
				move(0, 0);
			}
		}while(!done);

		/*** cards have been selected, now throw them in the crib ***/
		for(n = 0; n < numcardspicked; n++){
			hand.passcard(&crib, card_picked[n]);
			/** IMPORTANT
			this additional loop needs to be here, because
			the cards get rearranged in the hand when one is removed;
			**/

			for(m = n + 1; m < numcardspicked; m++)
				if(card_picked[m] > card_picked[n]) card_picked[m]--;
		}
	}
}

void playertype::divide_by_computer(void){
	int combo, max, n, numbits, bestcombo, bestscore = -1, score;
	handtype testhand;

	max = 1 << hand.numcards;

	for(combo = 0; combo < max; combo++){
		numbits = 0;
		for(n = 0; n < hand.numcards; n++){
			if(combo & (1 << n)) numbits++;
		}
		if(numbits == add_to_crib_size){
			/***Here is where we check the value of the hand ***/
			testhand.erase();

			/** generate the new hand **/
			for(n = 0; n < hand.numcards; n++){
				if(!(combo & (1 << n))){
					testhand += hand.card[n];
				}
			}

			/** get the value of it **/
			score = count_points(testhand, NOCUT, none);
			if(score > bestscore ){
				bestscore = score;
				bestcombo = combo;
			}
		}
	}

	/*** 
	very important:  This loop MUST go in this order because the deck
	moves all the cards after the one pulled to fill in the space.  If
	They are grabbed in ascending order, everything after the first one
	will be wrong.
	***/
	for(n = hand.numcards - 1; n >= 0; n--)
		if((1 << n) & bestcombo)
			hand.passcard(&crib, n);
}

