#include <stdio.h>
#include <string.h>
#include <ncurses.h>
#include <stdlib.h>
#include <stdio.h>
#include <ncurses.h>

/**
	xsize == 55
	ysize == 13
**/

class holetype{
    public:
	int x, y;

	holetype(){
		x = y = 0;
	}

	void set(int newy, int newx){
		x = newx;
		y = newy;
	}
};


class boardtype{
    public:
	int handsize, add_to_crib_size, starting_handsize, starting_cribsize;
        int xsize, ysize, numholes, numplayers, numcolourpairs, *pegcolour;
	char **image, **colour, **colourpair;
	holetype **hole, endhole;

	boardtype(){
		xsize = ysize = numholes = numplayers = numcolourpairs = 0;
		image = colour = colourpair = (char **)NULL;
		hole = NULL;
		pegcolour = (int *)NULL;
	}

	boardtype(int newysize, int newxsize, int length, int newnumplayers, int newnumcolourpairs){
		initialize(newysize, newxsize, length, newnumplayers, newnumcolourpairs);
	}

	void initialize(int newysize, int newxsize, int length, int newnumplayers, int newnumcolourpairs){

		int n;
		ysize = newysize;
		xsize = newxsize;

		pegcolour = new int[newnumplayers];
		image = new char*[newysize];
		colour = new char*[newysize];
		colourpair = new char*[newnumcolourpairs];

		numcolourpairs = newnumcolourpairs;
		for(n = 0; n < numcolourpairs; n++){
			colourpair[n] = new char[2];
		}

		for(n = 0; n < newysize; n++){
			image[n] = new char[newxsize + 1];
			colour[n] = new char[newxsize + 1];
			memset(image[n], ' ', (newxsize + 1) * sizeof(unsigned char));
			memset(colour[n], 0, (newxsize + 1) * sizeof(char));
		}

		numholes = length;
		numplayers = newnumplayers;

		hole = new holetype*[numplayers];
		for(n = 0; n < numplayers; n++){
			hole[n] = new holetype[numholes];
		}

	}

	~boardtype(){
		delete hole;
		delete image;
		delete colour;
		delete colourpair;
		delete pegcolour;
	}

	int save(char *filename){
		FILE *fout;
		int n, m;

		fout = fopen(filename, "wb");
		if(fout != NULL){
			fwrite((void *)&handsize, sizeof(int), 1, fout);
			fwrite((void *)&add_to_crib_size, sizeof(int), 1, fout);
			fwrite((void *)&starting_handsize, sizeof(int), 1, fout);
			fwrite((void *)&starting_cribsize, sizeof(int), 1, fout);
			fwrite((void *)&xsize, sizeof(int), 1, fout);
			fwrite((void *)&ysize, sizeof(int), 1, fout);
			fwrite((void *)&numholes, sizeof(int), 1, fout);
			fwrite((void *)&numplayers, sizeof(int), 1, fout);
			fwrite((void *)&numcolourpairs, sizeof(int), 1, fout);

			for(n = 0; n < ysize; n++)
				fwrite((void *)image[n], sizeof(char), xsize + 1, fout);
			for(n = 0; n < ysize; n++)
				fwrite((void *)colour[n], sizeof(char), xsize + 1, fout);
			for(n = 0; n < numcolourpairs; n++){
				fwrite((void *)colourpair[n], sizeof(char), 2, fout);
			}

			for(m = 0; m < numplayers; m++){
				for(n = 0; n < numholes; n++){
					fputc((int)hole[m][n].x, fout);
					fputc((int)hole[m][n].y, fout);
				}
				fputc((int)pegcolour[m], fout);
			}
			fputc((int)endhole.x, fout);
			fputc((int)endhole.y, fout);

			fclose(fout);

		}else{
			printf("Error opening save file.\n");
			return 1;
		}
		return 0;
	}

	int load(char *filename){
		FILE *fin;
		int n, m;
		int xs, ys, nh, np, nc;

		fin = fopen(filename, "rb");
		if(fin != NULL){
			fread((void *)&handsize, sizeof(int), 1, fin);
			fread((void *)&add_to_crib_size, sizeof(int), 1, fin);
			fread((void *)&starting_handsize, sizeof(int), 1, fin);
			fread((void *)&starting_cribsize, sizeof(int), 1, fin);
			fread((void *)&xs, sizeof(int), 1, fin);
			fread((void *)&ys, sizeof(int), 1, fin);
			fread((void *)&nh, sizeof(int), 1, fin);
			fread((void *)&np, sizeof(int), 1, fin);
			fread((void *)&nc, sizeof(int), 1, fin);

			initialize(ys, xs, nh, np, nc);

			for(n = 0; n < ysize; n++)
				fread((void *)image[n], sizeof(char), xsize + 1, fin);
			for(n = 0; n < ysize; n++)
				fread((void *)colour[n], sizeof(char), xsize + 1, fin);
			for(n = 0; n < numcolourpairs; n++){
				fread((void *)colourpair[n], sizeof(char), 2, fin);
			}

			for(m = 0; m < numplayers; m++){
				for(n = 0; n < numholes; n++){
					hole[m][n].x = fgetc(fin);
					hole[m][n].y = fgetc(fin);
				}
				pegcolour[m] = fgetc(fin);
			}
			endhole.x = fgetc(fin);
			endhole.y = fgetc(fin);

			fclose(fin);

		}else{
			printf("Error opening load file.\n");
			return 1;
		}
		return 0;
	}

	void makeholes(void);
};

int main(void){
	int n, m;
	boardtype board(12, 55, 120, 3, 7);

	strcpy(board.image[0],     ".....|.....|.....|.....|.....|.....|.....|.....|. . .  ");
	strcpy(board.image[1],     ".....|.....|.....|.....|.....|.....|.....|.....|. ..   ");
	strcpy(board.image[2],     ".....|.....|.....|.....|.....|.....|.....|.....|... . .");
	strcpy(board.image[3],     "     5     10    15    20    25    30    35    40 .    ");
	strcpy(board.image[4],     " .  .|.....|.....|.....|.....|.....|.....|         . ..");
	strcpy(board.image[5],     "  . .|.....|.....|.....|.....|.....|.....|  .    45----");
	strcpy(board.image[6],     "   ..|.....|.....|.....|.....|.....|.....|         . ..");
	strcpy(board.image[7],     ".. . S     95   100   105   110   115   120       .    ");
	strcpy(board.image[8],     "   ..|.....|.....|.....|.....|.....|.....|.....|... . .");
	strcpy(board.image[9],     "  . .|.....|.....|.....|.....|.....|.....|.....|. ..   ");
	strcpy(board.image[10],    " .  .|.....|.....|.....|.....|.....|.....|.....|. . .  ");
	strcpy(board.image[11],    "     85    80    75    70    65    60    55    50      ");

	strcpy(board.colour[0],  "1111111111111111111111111111111111111111111111111111100");
	strcpy(board.colour[1],  "3333333333333333333333333333333333333333333333333333110");
	strcpy(board.colour[2],  "2222222222222222222222222222222222222222222222222223311");
	strcpy(board.colour[3],  "0000000000000000000000000000000000000000000000000022331");
	strcpy(board.colour[4],  "0111111111111111111111111111111111111111111111100002331");
	strcpy(board.colour[5],  "1133333333333333333333333333333333333333333333300002331");
	strcpy(board.colour[6],  "1332222222222222222222222222222222222222222222200002331");
	strcpy(board.colour[7],  "1322000000000000000000000000000000000000000000000022331");
	strcpy(board.colour[8],  "1332222222222222222222222222222222222222222222222223311");
	strcpy(board.colour[9],  "1133333333333333333333333333333333333333333333333333110");
	strcpy(board.colour[10], "0111111111111111111111111111111111111111111111111111100");
	strcpy(board.colour[11], "0000000000000000000000000000000000000000000000000000000");

	board.colourpair[0][0] = COLOR_BLACK; board.colourpair[0][1] = COLOR_WHITE;
	board.colourpair[1][0] = COLOR_BLACK; board.colourpair[1][1] = COLOR_RED;
	board.colourpair[2][0] = COLOR_BLACK; board.colourpair[2][1] = COLOR_BLUE;
	board.colourpair[3][0] = COLOR_BLACK; board.colourpair[3][1] = COLOR_GREEN;
	board.colourpair[4][0] = COLOR_CYAN; board.colourpair[4][1] = COLOR_RED;
	board.colourpair[5][0] = COLOR_YELLOW; board.colourpair[5][1] = COLOR_BLUE;
	board.colourpair[6][0] = COLOR_MAGENTA; board.colourpair[6][1] = COLOR_GREEN;

	board.pegcolour[0] = 4;
	board.pegcolour[1] = 5;
	board.pegcolour[2] = 6;
	
	for(m = 0; m < board.ysize; m++){
		for(n = 0; n < board.xsize; n++){
			board.colour[m][n] -= '0';
		}
	}

	board.handsize = 4;
	board.add_to_crib_size = 1;
	board.starting_handsize = 5;
	board.starting_cribsize = 1;
	board.makeholes();

	board.save("3player.board");
	return 0;
}

void boardtype::makeholes(void){

/****RED*****/

	/************ first row (1 through 40) ***********/
	hole[0][0].set(0, 0); hole[0][1].set(0, 1); hole[0][2].set(0, 2); hole[0][3].set(0, 3); hole[0][4].set(0, 4);
	hole[0][5].set(0, 6); hole[0][6].set(0, 7); hole[0][7].set(0, 8); hole[0][8].set(0, 9); hole[0][9].set(0, 10);
	hole[0][10].set(0, 12); hole[0][11].set(0, 13); hole[0][12].set(0, 14); hole[0][13].set(0, 15); hole[0][14].set(0, 16);
	hole[0][15].set(0, 18); hole[0][16].set(0, 19); hole[0][17].set(0, 20); hole[0][18].set(0, 21); hole[0][19].set(0, 22);
	hole[0][20].set(0, 24); hole[0][21].set(0, 25); hole[0][22].set(0, 26); hole[0][23].set(0, 27); hole[0][24].set(0, 28);
	hole[0][25].set(0, 30); hole[0][26].set(0, 31); hole[0][27].set(0, 32); hole[0][28].set(0, 33); hole[0][29].set(0, 34);
	hole[0][30].set(0, 36); hole[0][31].set(0, 37); hole[0][32].set(0, 38); hole[0][33].set(0, 39); hole[0][34].set(0, 40);
	hole[0][35].set(0, 42); hole[0][36].set(0, 43); hole[0][37].set(0, 44); hole[0][38].set(0, 45); hole[0][39].set(0, 46);

	/********* round the bend on the right (41 through 50) ********/
	hole[0][40].set(0, 48); hole[0][41].set(0, 50); hole[0][42].set(0, 52); hole[0][43].set(2, 54); hole[0][44].set(4, 54);
	hole[0][45].set(6, 54); hole[0][46].set(8, 54); hole[0][47].set(10, 52); hole[0][48].set(10, 50); hole[0][49].set(10, 48);

	/********* bottom row (51 through 85) *********/
	hole[0][50].set(10, 46); hole[0][51].set(10, 45); hole[0][52].set(10, 44); hole[0][53].set(10, 43); hole[0][54].set(10, 42);
	hole[0][55].set(10, 40); hole[0][56].set(10, 39); hole[0][57].set(10, 38); hole[0][58].set(10, 37); hole[0][59].set(10, 36);
	hole[0][60].set(10, 34); hole[0][61].set(10, 33); hole[0][62].set(10, 32); hole[0][63].set(10, 31); hole[0][64].set(10, 30);
	hole[0][65].set(10, 28); hole[0][66].set(10, 27); hole[0][67].set(10, 26); hole[0][68].set(10, 25); hole[0][69].set(10, 24);
	hole[0][70].set(10, 22); hole[0][71].set(10, 21); hole[0][72].set(10, 20); hole[0][73].set(10, 19); hole[0][74].set(10, 18);
	hole[0][75].set(10, 16); hole[0][76].set(10, 15); hole[0][77].set(10, 14); hole[0][78].set(10, 13); hole[0][79].set(10, 12);
	hole[0][80].set(10, 10); hole[0][81].set(10, 9); hole[0][82].set(10, 8); hole[0][83].set(10, 7); hole[0][84].set(10, 6);

	/********* round the bend on the left (86 through 90) ********/
	hole[0][85].set(10, 4); hole[0][86].set(10, 1); hole[0][87].set(7, 0); hole[0][88].set(4, 1); hole[0][89].set(4, 4);

	/********* the last stretch (91 through 120) ******/
	hole[0][90].set(4, 6); hole[0][91].set(4, 7); hole[0][92].set(4, 8); hole[0][93].set(4, 9); hole[0][94].set(4, 10);
	hole[0][95].set(4, 12); hole[0][96].set(4, 13); hole[0][97].set(4, 14); hole[0][98].set(4, 15); hole[0][99].set(4, 16);
	hole[0][100].set(4, 18); hole[0][101].set(4, 19); hole[0][102].set(4, 20); hole[0][103].set(4, 21); hole[0][104].set(4, 22);
	hole[0][105].set(4, 24); hole[0][106].set(4, 25); hole[0][107].set(4, 26); hole[0][108].set(4, 27); hole[0][109].set(4, 28);
	hole[0][110].set(4, 30); hole[0][111].set(4, 31); hole[0][112].set(4, 32); hole[0][113].set(4, 33); hole[0][114].set(4, 34);
	hole[0][115].set(4, 36); hole[0][116].set(4, 37); hole[0][117].set(4, 38); hole[0][118].set(4, 39); hole[0][119].set(4, 40);

/****BLUE*****/

	/************ first row (1 through 40) ***********/
	hole[1][0].set(2, 0); hole[1][1].set(2, 1); hole[1][2].set(2, 2); hole[1][3].set(2, 3); hole[1][4].set(2, 4);
	hole[1][5].set(2, 6); hole[1][6].set(2, 7); hole[1][7].set(2, 8); hole[1][8].set(2, 9); hole[1][9].set(2, 10);
	hole[1][10].set(2, 12); hole[1][11].set(2, 13); hole[1][12].set(2, 14); hole[1][13].set(2, 15); hole[1][14].set(2, 16);
	hole[1][15].set(2, 18); hole[1][16].set(2, 19); hole[1][17].set(2, 20); hole[1][18].set(2, 21); hole[1][19].set(2, 22);
	hole[1][20].set(2, 24); hole[1][21].set(2, 25); hole[1][22].set(2, 26); hole[1][23].set(2, 27); hole[1][24].set(2, 28);
	hole[1][25].set(2, 30); hole[1][26].set(2, 31); hole[1][27].set(2, 32); hole[1][28].set(2, 33); hole[1][29].set(2, 34);
	hole[1][30].set(2, 36); hole[1][31].set(2, 37); hole[1][32].set(2, 38); hole[1][33].set(2, 39); hole[1][34].set(2, 40);
	hole[1][35].set(2, 42); hole[1][36].set(2, 43); hole[1][37].set(2, 44); hole[1][38].set(2, 45); hole[1][39].set(2, 46);

	/********* round the bend on the right (41 through 50) ********/
	hole[1][40].set(2, 48); hole[1][41].set(2, 49); hole[1][42].set(2, 50); hole[1][43].set(3, 50); hole[1][44].set(4, 51);
	hole[1][45].set(6, 51); hole[1][46].set(7, 50); hole[1][47].set(8, 50); hole[1][48].set(8, 49); hole[1][49].set(8, 48);


	/********* bottom row (51 through 85) *********/
	hole[1][50].set(8, 46); hole[1][51].set(8, 45); hole[1][52].set(8, 44); hole[1][53].set(8, 43); hole[1][54].set(8, 42);
	hole[1][55].set(8, 40); hole[1][56].set(8, 39); hole[1][57].set(8, 38); hole[1][58].set(8, 37); hole[1][59].set(8, 36);
	hole[1][60].set(8, 34); hole[1][61].set(8, 33); hole[1][62].set(8, 32); hole[1][63].set(8, 31); hole[1][64].set(8, 30);
	hole[1][65].set(8, 28); hole[1][66].set(8, 27); hole[1][67].set(8, 26); hole[1][68].set(8, 25); hole[1][69].set(8, 24);
	hole[1][70].set(8, 22); hole[1][71].set(8, 21); hole[1][72].set(8, 20); hole[1][73].set(8, 19); hole[1][74].set(8, 18);
	hole[1][75].set(8, 16); hole[1][76].set(8, 15); hole[1][77].set(8, 14); hole[1][78].set(8, 13); hole[1][79].set(8, 12);
	hole[1][80].set(8, 10); hole[1][81].set(8, 9); hole[1][82].set(8, 8); hole[1][83].set(8, 7); hole[1][84].set(8, 6);

	/********* round the bend on the left (86 through 90) ********/
	hole[1][85].set(8, 4); hole[1][86].set(8, 3); hole[1][87].set(7, 3); hole[1][88].set(6, 2); hole[1][89].set(6, 4);

	/********* the last stretch (91 through 120) ******/
	hole[1][90].set(6, 6); hole[1][91].set(6, 7); hole[1][92].set(6, 8); hole[1][93].set(6, 9); hole[1][94].set(6, 10);
	hole[1][95].set(6, 12); hole[1][96].set(6, 13); hole[1][97].set(6, 14); hole[1][98].set(6, 15); hole[1][99].set(6, 16); 
	hole[1][100].set(6, 18); hole[1][101].set(6, 19); hole[1][102].set(6, 20); hole[1][103].set(6, 21); hole[1][104].set(6, 22);
	hole[1][105].set(6, 24); hole[1][106].set(6, 25); hole[1][107].set(6, 26); hole[1][108].set(6, 27); hole[1][109].set(6, 28);
	hole[1][110].set(6, 30); hole[1][111].set(6, 31); hole[1][112].set(6, 32); hole[1][113].set(6, 33); hole[1][114].set(6, 34);
	hole[1][115].set(6, 36); hole[1][116].set(6, 37); hole[1][117].set(6, 38); hole[1][118].set(6, 39); hole[1][119].set(6, 40);

/****GREEN****/
	/*************** first row (1 through 40) ******************/
	hole[2][0].set(1, 0); hole[2][1].set(1, 1); hole[2][2].set(1, 2); hole[2][3].set(1, 3); hole[2][4].set(1, 4);
	hole[2][5].set(1, 6); hole[2][6].set(1, 7); hole[2][7].set(1, 8); hole[2][8].set(1, 9); hole[2][9].set(1, 10);
	hole[2][10].set(1, 12); hole[2][11].set(1, 13); hole[2][12].set(1, 14); hole[2][13].set(1, 15); hole[2][14].set(1, 16);
	hole[2][15].set(1, 18); hole[2][16].set(1, 19); hole[2][17].set(1, 20); hole[2][18].set(1, 21); hole[2][19].set(1, 22);
	hole[2][20].set(1, 24); hole[2][21].set(1, 25); hole[2][22].set(1, 26); hole[2][23].set(1, 27); hole[2][24].set(1, 28);
	hole[2][25].set(1, 30); hole[2][26].set(1, 31); hole[2][27].set(1, 32); hole[2][28].set(1, 33); hole[2][29].set(1, 34);
	hole[2][30].set(1, 36); hole[2][31].set(1, 37); hole[2][32].set(1, 38); hole[2][33].set(1, 39); hole[2][34].set(1, 40);
	hole[2][35].set(1, 42); hole[2][36].set(1, 43); hole[2][37].set(1, 44); hole[2][38].set(1, 45); hole[2][39].set(1, 46);

	/********* round the bend on the right (41 through 50) ********/
	hole[2][40].set(1, 48); hole[2][41].set(1, 50); hole[2][42].set(1, 51); hole[2][43].set(2, 52); hole[2][44].set(4, 53);
	hole[2][45].set(6, 53); hole[2][46].set(8, 52); hole[2][47].set(9, 51); hole[2][48].set(9, 50); hole[2][49].set(9, 48);

	/********* bottom row (51 through 85) *********/
	hole[2][50].set(9, 46); hole[2][51].set(9, 45); hole[2][52].set(9, 44); hole[2][53].set(9, 43); hole[2][54].set(9, 42);
	hole[2][55].set(9, 40); hole[2][56].set(9, 39); hole[2][57].set(9, 38); hole[2][58].set(9, 37); hole[2][59].set(9, 36);
	hole[2][60].set(9, 34); hole[2][61].set(9, 33); hole[2][62].set(9, 32); hole[2][63].set(9, 31); hole[2][64].set(9, 30);
	hole[2][65].set(9, 28); hole[2][66].set(9, 27); hole[2][67].set(9, 26); hole[2][68].set(9, 25); hole[2][69].set(9, 24);
	hole[2][70].set(9, 22); hole[2][71].set(9, 21); hole[2][72].set(9, 20); hole[2][73].set(9, 19); hole[2][74].set(9, 18);
	hole[2][75].set(9, 16); hole[2][76].set(9, 15); hole[2][77].set(9, 14); hole[2][78].set(9, 13); hole[2][79].set(9, 12);
	hole[2][80].set(9, 10); hole[2][81].set(9, 9); hole[2][82].set(9, 8); hole[2][83].set(9, 7); hole[2][84].set(9, 6);

	/********* round the bend on the left (86 through 90) ********/
	hole[2][85].set(9, 4); hole[2][86].set(9, 2); hole[2][87].set(7, 1); hole[2][88].set(5, 2); hole[2][89].set(5, 4);

	/********* the last stretch (91 through 120) ******/
	hole[2][90].set(5, 6); hole[2][91].set(5, 7); hole[2][92].set(5, 8); hole[2][93].set(5, 9); hole[2][94].set(5, 10);
	hole[2][95].set(5, 12); hole[2][96].set(5, 13); hole[2][97].set(5, 14); hole[2][98].set(5, 15); hole[2][99].set(5, 16); 
	hole[2][100].set(5, 18); hole[2][101].set(5, 19); hole[2][102].set(5, 20); hole[2][103].set(5, 21); hole[2][104].set(5, 22);
	hole[2][105].set(5, 24); hole[2][106].set(5, 25); hole[2][107].set(5, 26); hole[2][108].set(5, 27); hole[2][109].set(5, 28);
	hole[2][110].set(5, 30); hole[2][111].set(5, 31); hole[2][112].set(5, 32); hole[2][113].set(5, 33); hole[2][114].set(5, 34);
	hole[2][115].set(5, 36); hole[2][116].set(5, 37); hole[2][117].set(5, 38); hole[2][118].set(5, 39); hole[2][119].set(5, 40);


	endhole.set(5, 44);
}
